@@ -68,7 +68,6 @@ def train(params, train_set, num_boost_round=100,
         For multi-class task, the preds is group by class_id first, then group by row_id.
         If you want to get i-th row preds in j-th class, the access way is score[j * num_data + i]
         and you should group grad and hess in this way as well.
-
     feval : callable, list of callable functions or None, optional (default=None)
         Customized evaluation function.
         Each evaluation function should accept two parameters: preds, train_data,
@@ -115,7 +114,6 @@ def train(params, train_set, num_boost_round=100,
         if early stopping logic is enabled by setting ``early_stopping_rounds``.
     evals_result: dict or None, optional (default=None)
         This dictionary used to store all evaluation results of all the items in ``valid_sets``.
-
         .. rubric:: Example
 
         With a ``valid_sets`` = [valid_set, train_set],
@@ -154,6 +152,14 @@ def train(params, train_set, num_boost_round=100,
     booster : Booster
         The trained Booster model.
 
+    Example
+    -------
+    >>> gp_model = gpb.GPModel(group_data=group, likelihood="gaussian")
+    >>> data_train = gpb.Dataset(X, y)
+    >>> params = {'objective': 'regression_l2', 'verbose': 0}
+    >>> bst = gpb.train(params=params, train_set=data_train,  gp_model=gp_model,
+    >>>                 num_boost_round=100)
+
     :Authors:
         Authors of the LightGBM Python package
         Fabio Sigrist
@@ -240,7 +246,7 @@ def train(params, train_set, num_boost_round=100,
         params['train_gp_model_cov_pars'] = train_gp_model_cov_pars
         # Set the default metric to the (approximate marginal) negative log-likelihood if only the training loss should be calculated
         if is_valid_contain_train and len(reduced_valid_sets) == 0 and params.get('metric') is None:
-            if gp_model.get_likelihood_name() == "gaussian":
+            if gp_model._get_likelihood_name() == "gaussian":
                 params['metric'] = "neg_log_likelihood"
             else:
                 params['metric'] = "approx_neg_marginal_log_likelihood"
@@ -459,9 +465,11 @@ def _make_n_folds(full_data, folds, nfold, params, seed, gp_model=None, use_gp_m
             cov_fct_shape = gp_model.cov_fct_shape
             cov_fct_taper_range = gp_model.cov_fct_taper_range
             ind_effect_group_rand_coef = gp_model.ind_effect_group_rand_coef
+            drop_intercept_group_rand_effect = gp_model.drop_intercept_group_rand_effect
             gp_model_train = GPModel(group_data=group_data,
                                      group_rand_coef_data=group_rand_coef_data,
                                      ind_effect_group_rand_coef=ind_effect_group_rand_coef,
+                                     drop_intercept_group_rand_effect=drop_intercept_group_rand_effect,
                                      gp_coords=gp_coords,
                                      gp_rand_coef_data=gp_rand_coef_data,
                                      cov_function=cov_function,
@@ -470,22 +478,22 @@ def _make_n_folds(full_data, folds, nfold, params, seed, gp_model=None, use_gp_m
                                      vecchia_approx=vecchia_approx,
                                      num_neighbors=num_neighbors,
                                      vecchia_ordering=vecchia_ordering,
-                                     vecchia_pred_type=vecchia_pred_type,
-                                     num_neighbors_pred=num_neighbors_pred,
                                      cluster_ids=cluster_ids,
-                                     likelihood=gp_model.get_likelihood_name(),
+                                     likelihood=gp_model._get_likelihood_name(),
                                      free_raw_data=True)
             if use_gp_model_for_validation:
                 gp_model_train.set_prediction_data(group_data_pred=group_data_pred,
                                                    group_rand_coef_data_pred=group_rand_coef_data_pred,
                                                    gp_coords_pred=gp_coords_pred,
                                                    gp_rand_coef_data_pred=gp_rand_coef_data_pred,
-                                                   cluster_ids_pred=cluster_ids_pred)
+                                                   cluster_ids_pred=cluster_ids_pred,
+                                                   vecchia_pred_type=vecchia_pred_type,
+                                                   num_neighbors_pred=num_neighbors_pred,
+                                                   cg_delta_conv_pred=gp_model.cg_delta_conv_pred)
             cvbooster = Booster(params=tparam, train_set=train_set, gp_model=gp_model_train)
-            gp_model.set_likelihood(
-                gp_model_train.get_likelihood_name())  # potentially change likelihood in case this was done in the booster to reflect implied changes in the default optimizer for different likelihoods
-            gp_model_train.set_optim_params(params=gp_model.get_optim_params())
-            gp_model_train.set_optim_coef_params(params=gp_model.get_optim_params())
+            gp_model._set_likelihood(
+                gp_model_train._get_likelihood_name())  # potentially change likelihood in case this was done in the booster to reflect implied changes in the default optimizer for different likelihoods
+            gp_model_train.set_optim_params(params=gp_model._get_optim_params())
         else:  # no gp_model
             cvbooster = Booster(tparam, train_set)
         if eval_train_metric:
@@ -579,7 +587,6 @@ def cv(params, train_set, num_boost_round=100,
         For multi-class task, the preds is group by class_id first, then group by row_id.
         If you want to get i-th row preds in j-th class, the access way is score[j * num_data + i]
         and you should group grad and hess in this way as well.
-
     feval : callable, list of callable functions or None, optional (default=None)
         Customized evaluation function.
         Each evaluation function should accept two parameters: preds, train_data,
@@ -654,13 +661,22 @@ def cv(params, train_set, num_boost_round=100,
         ...}.
         If ``return_cvbooster=True``, also returns trained boosters via ``cvbooster`` key.
 
+    Example
+    -------
+    >>> gp_model = gpb.GPModel(group_data=group, likelihood="gaussian")
+    >>> data_train = gpb.Dataset(X, y)
+    >>> params = {'objective': 'regression_l2', 'verbose': 0}
+    >>> cvbst = gpb.cv(params=params, train_set=data_train,
+    >>>                gp_model=gp_model, use_gp_model_for_validation=True,
+    >>>                num_boost_round=1000, early_stopping_rounds=5,
+    >>>                nfold=4, verbose_eval=True, show_stdv=False, seed=1)
+
     :Authors:
         Authors of the LightGBM Python package
         Fabio Sigrist
     """
     if fit_GP_cov_pars_OOS:
-        raise ValueError("The GPBoostOOS algorithm (fit_GP_cov_pars_OOS=True) is currently not supported in Python. "
-                         "If you need this feature, contact the developer of this package or open a GitHub issue.")
+        raise ValueError("The GPBoostOOS algorithm (fit_GP_cov_pars_OOS=True) is not yet implemented in Python.")
     if not isinstance(train_set, Dataset):
         raise TypeError("cv only accepts Dataset objects as train_set")
     if train_set.free_raw_data:
@@ -954,6 +970,28 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
         The dictionary has the following format:
         {'best_params': best_params, 'best_num_boost_round': best_num_boost_round, 'best_score': best_score}
 
+    Example
+    -------
+    >>> param_grid = {'learning_rate': [1,0.1,0.01], 'min_data_in_leaf': [1,10,100],
+    >>>                     'max_depth': [1,3,5,10,-1]}
+    >>> gp_model = gpb.GPModel(group_data=group, likelihood="gaussian")
+    >>> data_train = gpb.Dataset(X, y)
+    >>> params = {'objective': 'regression_l2', 'verbose': 0}
+    >>> opt_params = gpb.grid_search_tune_parameters(param_grid=param_grid,
+    >>>                                              params=params,
+    >>>                                              num_try_random=None,
+    >>>                                              nfold=4,
+    >>>                                              gp_model=gp_model,
+    >>>                                              use_gp_model_for_validation=True,
+    >>>                                              train_set=data_train,
+    >>>                                              verbose_eval=1,
+    >>>                                              num_boost_round=1000,
+    >>>                                              early_stopping_rounds=10,
+    >>>                                              seed=1000)
+    >>> print("Best parameters: " + str(opt_params['best_params']))
+    >>> print("Best number of iterations: " + str(opt_params['best_iter']))
+    >>> print("Best score: " + str(opt_params['best_score']))
+
     :Authors:
         Fabio Sigrist
     """
@@ -1044,7 +1082,10 @@ def grid_search_tune_parameters(param_grid, train_set, params=None, num_try_rand
             else:
                 best_num_boost_round = np.argmin(cvbst[next(iter(cvbst))])
             if verbose_eval >= 1:
-                print("***** New best score (" + str(best_score) + ") found for the following parameter combination:")
+                metric_name = list(cvbst.keys())[0]
+                metric_name = metric_name.split('-mean', 1)[0]
+                print("***** New best test score ("+metric_name+" = " + str(best_score) +
+                      ") found for the following parameter combination:")
                 best_params_print = copy.deepcopy(best_params)
                 best_params_print['num_boost_round'] = best_num_boost_round
                 print(best_params_print)
