@@ -87,13 +87,25 @@ namespace LightGBM {
 						const REModel* re_model = objective->GetGPModel();
 						if (re_model->GaussLikelihood()) {//Gaussian data (this is rarely used)
 							std::vector<double> minus_gp_pred(num_data_);
-							re_model->Predict(nullptr, num_data_, minus_gp_pred.data(),
-								false, false, false,
-								nullptr, nullptr, nullptr,
-								nullptr, nullptr,
-								nullptr, nullptr,
-								true, nullptr, -1,
-								nullptr, nullptr,
+							re_model->Predict(nullptr,
+								num_data_,
+								minus_gp_pred.data(),
+								false,
+								false,
+								false,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								true,
+								nullptr,
+								-1,
+								-1.,
+								nullptr,
+								nullptr,
 								true);//suppress_calc_cov_factor=true as this has been done already at the end of the last boosting update iteration
 							// Note that the re_model already has the updated response data score - label = F_t - y 
 							//	since 'Boosting()' is called (i.e. gradients are calculated) at the end of TrainOneIter()
@@ -105,13 +117,25 @@ namespace LightGBM {
 						}//end Gaussian data
 						else {//non-Gaussian data
 							std::vector<double> gp_pred(num_data_);
-							re_model->Predict(nullptr, num_data_, gp_pred.data(),
-								false, false, true,
-								nullptr, nullptr, nullptr,
-								nullptr, nullptr,
-								nullptr, nullptr,
-								true, nullptr, -1,
-								nullptr, score,
+							re_model->Predict(nullptr,
+								num_data_,
+								gp_pred.data(),
+								false,
+								false,
+								true,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								nullptr,
+								true,
+								nullptr,
+								-1,
+								-1.,
+								nullptr,
+								score,
 								true);//suppress_calc_cov_factor=true as this has been done already at the end of the last boosting update iteration
 							// Note that the re_model already has the updated training score (= F_t)
 							//	since 'Boosting()' is called (i.e. gradients are calculated) at the end of TrainOneIter()
