@@ -1,6 +1,7 @@
 context("GPBoost_combined_boosting_GP_random_effects")
 
 TOLERANCE <- 1E-3
+TOLERANCE2 <- 1E-2
 DEFAULT_OPTIM_PARAMS <- list(optimizer_cov="fisher_scoring", delta_rel_conv=1E-6)
 
 # Function that simulates uniform random variables
@@ -402,8 +403,22 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                    objective = "regression_l2",
                    verbose = 0,
                    leaves_newton_update = FALSE)
-    cov_pars <- c(0.005087127, 0.590528487, 0.390569351)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE2)
+    
+    # Use Adam for training
+    gp_model <- GPModel(group_data = group_data_train)
+    gp_model$set_optim_params(params = list(optimizer_cov="adam"))
+    bst <- gpboost(data = X_train,
+                   label = y_train,
+                   gp_model = gp_model,
+                   nrounds = 62,
+                   learning_rate = 0.01,
+                   max_depth = 6,
+                   min_data_in_leaf = 5,
+                   objective = "regression_l2",
+                   verbose = 0,
+                   leaves_newton_update = FALSE)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE2)
   })
   
   
